package jfxtras.labs.scene.layout;

import javafx.scene.Node;

class GenericLayoutConstraints
{
	/**
	 * The layout constraints
	 *
	 */
	public static class C<T>
	{
		// minWidth
		public T minWidth(double value) { this.minWidth = value; return (T)this; }
		double minWidth = -1;
		double minWidthReset = UNKNOWN;
		
		// prefWidth
		public T prefWidth(double value) { this.prefWidth = value; return (T)this; }
		double prefWidth = -1;
		double prefWidthReset = UNKNOWN;
		
		// maxWidth
		public T maxWidth(double value) { this.maxWidth = value; return (T)this; }
		double maxWidth = -1;
		double maxWidthReset = UNKNOWN;

		// minHeight
		public T minHeight(double value) { this.minHeight = value; return (T)this; }
		double minHeight = -1;
		double minHeightReset = UNKNOWN;

		// prefHeight
		public T prefHeight(double value) { this.prefHeight = value; return (T)this; }
		double prefHeight = -1;
		double prefHeightReset = UNKNOWN;
		
		// maxHeight
		public T maxHeight(double value) { this.maxHeight = value; return (T)this; }
		double maxHeight = -1;
		double maxHeightReset = UNKNOWN;
		
		/**
		 * @param node
		 */
		protected void rememberResetValues(Node node)
		{
			if (node instanceof javafx.scene.control.Control)
			{
				javafx.scene.control.Control lControl = (javafx.scene.control.Control)node;
				
				// setup the reset values on the first apply
				if (minWidthReset == UNKNOWN) minWidthReset = lControl.getMinWidth();
				if (prefWidthReset == UNKNOWN) prefWidthReset = lControl.getPrefWidth();
				if (maxWidthReset == UNKNOWN) maxWidthReset = lControl.getMaxWidth();
				if (minHeightReset == UNKNOWN) minHeightReset = lControl.getMinHeight();
				if (prefHeightReset == UNKNOWN) prefHeightReset = lControl.getPrefHeight();
				if (maxHeightReset == UNKNOWN) maxHeightReset = lControl.getMaxHeight();
			}
		}
		
		/**
		 * @param node
		 */
		protected void apply(Node node)
		{
			if (node instanceof javafx.scene.control.Control)
			{
				javafx.scene.control.Control lControl = (javafx.scene.control.Control)node;
				
				// setup the reset values on the first apply
				rememberResetValues(lControl);
				
				// either set or reset values
				lControl.setMinWidth(minWidth >= 0 ? minWidth : minWidthReset);
				lControl.setPrefWidth(prefWidth >= 0 ? prefWidth : prefWidthReset);
				lControl.setMaxWidth(maxWidth >= 0 ? maxWidth : maxWidthReset);
				lControl.setMinHeight(minHeight >= 0 ? minHeight : minHeightReset);
				lControl.setPrefHeight(prefHeight >= 0 ? prefHeight : prefHeightReset);
				lControl.setMaxHeight(maxHeight >= 0 ? maxHeight : maxHeightReset);
			}
		}
		
	}
	static final double UNKNOWN = -1234;
	
	/**
	 * 
	 */
	static public void overrideMaxWidth(Node node, C c)
	{
		// just to prevent problems
		if (node == null) return;
		
		// make things go away
		if ( node instanceof javafx.scene.control.Button
		  || node instanceof javafx.scene.control.ToggleButton
		  || node instanceof javafx.scene.control.CheckBox
		  || node instanceof javafx.scene.control.RadioButton
		  || node instanceof javafx.scene.control.ChoiceBox
		   )
		{
			javafx.scene.control.Control lControl = (javafx.scene.control.Control)node;
			lControl.setMaxWidth( c.maxWidth >= 0 ? c.maxWidth : Double.MAX_VALUE);
		}
	}
	
	/**
	 * 
	 */
	static public void overrideMaxHeight(Node node, C c)
	{
		// just to prevent problems
		if (node == null) return;
		
		// make things go away
		if ( node instanceof javafx.scene.control.Button
		  || node instanceof javafx.scene.control.ToggleButton
		   )
		{
			javafx.scene.control.Control lControl = (javafx.scene.control.Control)node;
			lControl.setMaxHeight( c.maxHeight >= 0 ? c.maxHeight : Double.MAX_VALUE);
		}
	}
	
}
